/**
 * @typedef {'ALL_FORM_FACTORS' | 'PHONE' | 'DESKTOP' | 'TABLET'} CruxApiFormFactor
 * @typedef {'offline' | 'slow-2G' | '2G' | '3G' | '4G'} CruxApiConnection
 * @typedef {{ histogram: { start: number | string, end: number | string, density: number }[], percentiles: { p75: number | string } }} CruxApiMetric
 * @typedef {{
 *    record: {
 *      key: {
 *        url: string,
 *        effectiveConnectionType?: CruxApiConnection,
 *        formFactor?:CruxApiFormFactor
 *      },
 *      metrics: {
 *        first_contentful_paint?: CruxApiMetric,
 *        largest_contentful_paint?: CruxApiMetric,
 *        first_input_delay?: CruxApiMetric,
 *        cumulative_layout_shift?: CruxApiMetric,
 *      }
 *    },
 *    urlNormalizationDetails?: {
 *      originalUrl: string,
 *      normalizedUrl: string
 *    }
 * }} CruxApiResponse
 */
/**
 * Fetch CrUX API.
 * Inspired by: https://github.com/GoogleChrome/CrUX/blob/master/js/crux-api-util.js
 *
 * Handle errors:
 * {
 *   "code": 404,
 *   "message": "chrome ux report data not found",
 *   "status": "NOT_FOUND"
 * }
 * {
 *   "code":429,
 *   "message":"Quota exceeded for quota group 'default' and limit 'Queries per 100 seconds' of service 'chromeuxreport.googleapis.com' for consumer 'project_number:224801012400'.",
 *   "status":"RESOURCE_EXHAUSTED",
 *   "details":[{"@type":"type.googleapis.com/google.rpc.Help","links":[{"description":"Google developer console API key","url":"https://console.developers.google.com/project/224801012400/apiui/credential"}]}]
 * }
 * @param {{ key: string, fetch?: any, maxRetries?: number, maxRetryTimeout?: number }} options
 */
export function createCruxApi(options: {
    key: string;
    fetch?: any;
    maxRetries?: number;
    maxRetryTimeout?: number;
}): (params: {
    url?: string;
    origin?: string;
    formFactor?: CruxApiFormFactor;
    effectiveConnectionType?: CruxApiConnection;
}, retryCounter?: number) => Promise<CruxApiResponse | null>;
export type CruxApiFormFactor = "ALL_FORM_FACTORS" | "PHONE" | "DESKTOP" | "TABLET";
export type CruxApiConnection = "offline" | "slow-2G" | "2G" | "3G" | "4G";
export type CruxApiMetric = {
    histogram: {
        start: number | string;
        end: number | string;
        density: number;
    }[];
    percentiles: {
        p75: number | string;
    };
};
export type CruxApiResponse = {
    record: {
        key: {
            url: string;
            effectiveConnectionType?: CruxApiConnection;
            formFactor?: CruxApiFormFactor;
        };
        metrics: {
            first_contentful_paint?: CruxApiMetric;
            largest_contentful_paint?: CruxApiMetric;
            first_input_delay?: CruxApiMetric;
            cumulative_layout_shift?: CruxApiMetric;
        };
    };
    urlNormalizationDetails?: {
        originalUrl: string;
        normalizedUrl: string;
    };
};
